@@ -3,6 +3,7 @@ import json
 import logging
 
 from django.db import IntegrityError, transaction
+from django.db.models import Q
 
 from twisted.internet.threads import deferToThread
 
@@ -32,8 +33,11 @@ class DatabaseJobSource(object):
             if device.status != Device.IDLE:
                 return None
             jobs_for_device = TestJob.objects.all().filter(
-                target=device, status=TestJob.SUBMITTED)
-            jobs_for_device.order_by('submit_time')
+                Q(target=device) | Q(device_type=device.device_type),
+                status=TestJob.SUBMITTED)
+            jobs_for_device = jobs_for_device.extra(
+                select={'is_targeted': 'target_id is not NULL'},
+                order_by=['-is_targeted', 'submit_time'])
             jobs = jobs_for_device[:1]
             if jobs:
                 job = jobs[0]
@@ -56,6 +60,12 @@ class DatabaseJobSource(object):
                     transaction.commit()
                     return json.loads(job.definition)
             else:
+                # We don't really need to rollback here, as no modifying
+                # operations have been made to the database.  But Django is
+                # stupi^Wconservative and assumes the queries that have been
+                # issued might have been modifications.
+                # See https://code.djangoproject.com/ticket/16491.
+                transaction.rollback()
                 return None
 
     def getJobForBoard(self, board_name):
